###################################
# PROLOG SECTION
# bouncing_ball.py
#
# A Turtle Graphics program to display a bouncing ball in a
# graphics window. The ball is really a circular turtle that
# uses animation to move it around. The program has logic to
# detect that the ball is about to exit the window and make it
# "bounce" off the edge of the window back into the middle.
#
# Each time the ball hits the edge of the window, a beep sound is
# produced to indicate that it is bouncing back.
#
# To stop the moving ball, you just click with the mouse anywhere
# inside the graphics window.
#
# Tested with Python 3.7.9 on Windows 10
# Fri July 16, 2021
#
# Copyright ©. All rights reserved. NAF 2021.
#
# Possible future enhancements:
# Unresolved bugs:
###################################

###################################
# ENVIRONMENT SETUP SECTION
# Imports: turtle, random, winsound
###################################

# import the turtle routines
from turtle import *
# import the random number routines
from random import *
# import Windows sound routines
from winsound import *

###################################
# FUNCTION DEFINITION SECTION
# Definitions: stop_loop()
# (also used in Rotate Square)
###################################

#
# Function to handle a mouse click inside the graphics window.
#
# The function sets the "endloop" flag to True so
# the while-loop terminates and the ball stops bouncing.
#
def stop_loop(clickx,clicky):
    # use global to gain access to the endloop variable
    # in the main program so we can change its value
    global endloop
    # set the flag that cause the while-loop to end
    endloop = True
    return

###################################
# PROCESSING INITIALIZATION SECTION
###################################

#
# Set up the window.
#

# create the drawing window and
# set the graphics window dimensions
# to 600X400 centered at (0,0)
maxx = 300
maxy = 200
minx = -maxx
miny = -maxy
setup(width = 2 * maxx, height = 2 * maxy)

# set the window title
title("bouncing ball program")

# set the window background color
bgcolor('grey')

#
# Set up the turtle as a ball.
#

# create the turtle as a ball
ball=Turtle()
# hide the ball to begin with
ball.hideturtle()
# turn off drawing a line when the ball moves
# to see the ball's path, comment out this line
ball.penup()
# set the shape, size, and color of the ball
ball.shape("circle")
ball.shapesize(3,3,3)
ball.color("blue")
# distance from center of ball to edge of window.
# ball can't get any closer to the edge than this
# (at this point, the ball must "bounce").
bounce_point = 20

#
# Set up the ball to move.
#

# Pick a random starting location for the ball.
# Variables x and y are the x- and y-coordinates
# for the moving ball.
x = randrange(minx + bounce_point, maxx - bounce_point)
y = randrange(miny + bounce_point, maxy - bounce_point)

# Set the initial movement increments.
# This is how far the ball moves, in pixels
# each trip through the while-loop.
dx = 5
dy = 5

#
# Move the ball.
#

# Move the ball to its starting position.
# It is not visible yet.
ball.goto(x,y)

# Display the ball as it moves around the window.
ball.showturtle()

#
# Set up the mechanism to stop the ball and end the program.
#

# Allow a mouse click to stop the ball
# by calling the stop_loop() function.
# This sets the endloop variable to True
# which causes the main
# animation while-loop to terminate.
onscreenclick(stop_loop)

###################################
# PROCESSING SECTION
# Main loop that moves the bouncing ball
# until the mouse is clicked 
# inside the window.
# Branching code: if/else
# Looping code: while-loop
###################################

# Initialize the loop exit condition variable to false.
# This changes only when the mouse is clicked in the window.
endloop=False

# Main Loop:
# Keep moving the ball until
# the window is closed by 
# clicking inside it.
while not endloop:

    #
    # Calculate the possible new x coordinate for the ball by
    # adding the move increment to the current x-coordinate value
    #
    
    xx = x + dx

    #
    # If the new position is too close to the left hand
    # edge of the window, set it to move in the opposite
    # direction and make a beep.
    #
    if xx < minx + bounce_point:
        # Limit x to the left-hand window edge.
        xx = minx + bounce_point
        # Reverse the x direction the ball is moving.
        dx = -dx
        # Sound a beep.
        MessageBeep(-1)
    
    #    
    # Same with the right-hand window.
    # 
    if xx > maxx - bounce_point:
        # Limit x to the right-hand window edge.
        xx = maxx - bounce_point
        # Reverse the x direction the ball is moving.
        dx = -dx
        # Sound a beep.
        MessageBeep(-1)

    #    
    # Calculate the possible new y coordinate for the ball by
    # adding the move increment to the current y-coordinate value.
    #

    yy = y + dy

    #
    # Same with the y-coordinate and the top.
    # 
    if yy < miny + bounce_point:
        # limit y to the window top edge
        yy = miny + bounce_point
        # reverse the y direction the ball is moving
        dy = -dy
        # sound a beep
        MessageBeep(-1)

    #   
    # Same with the y-coordinate and the bottom.
    #
    if yy > maxy - bounce_point:
        # limit y to the bottom of the window
        yy = maxy - bounce_point
        # reverse the y direction
        dy = -dy
        # sound a beep
        MessageBeep(-1)

    #   
    # Set the new x and y coordinates for the center of the ball
    # using the values just calculated.
    #
    x = xx
    y = yy
    
    # Move the ball to its new location. This will cause the old
    # image of the ball in the window to be erased and a new one
    # to be drawn at its new position, thus producing an
    # animated view of the moving ball.
    
    ball.goto(x,y)

###################################   
# CLEANUP, TERMINATION, AND EXIT SECTION
###################################

# close the drawing window without user action
# (the done() function requires user action)
bye()
