###################################
# PROLOG SECTION
# rotating_square.py
# Draws a rotating square using Python Turtle Graphics
#
# Tested with Python 3.7.9 on Windows 10
# Fri July 16, 2021
#
# Copyright ©. All rights reserved. NAF 2021.
#
# Possible future enhancements:
# Unresolved bugs:
###################################

###################################
# ENVIRONMENT SETUP SECTION
# Imports: turtle
###################################

# import turtle drawing routine
from turtle import *

###################################
# FUNCTION DEFINITION SECTION
# Definitions: stop_loop()
# (also used in Bouncing Ball)
###################################

#
# Function to handle a mouse click inside the graphics window.
#
# The function sets the "endloop" flag to True so
# the while-loop terminates and the square stops rotating.
#
def stop_loop(clickx,clicky):
    # use global to gain access to the endloop variable
    # in the main program so we can change its value
    global endloop
    # set the flag that causes the while-loop to end
    endloop=True
    return

###################################
# PROCESSING INITIALIZATION SECTION
###################################

#
# Set up the window
#

# create the drawing window
setup()

# set the window title
title("rotating square program")

# set the window background color
bgcolor('grey')

#
# Set up the turtle as a square.
#

# create the turtle as a square
square = Turtle()
# set the shape and size of the square
square.shape("square")
square.shapesize(10,10,10)
# specify the various colors to be used for drawing the square
colors = ("red","blue","green","yellow","purple","black","pink")
# initialize the counter we use to set the square color
loop_count = 0

#
# Set up the square to move.
#
# set the starting point for the square
square.goto(0,0)

#
# Set up the mechanism to stop the square and end the program.
#
# end the program when the mouse is clicked
onscreenclick(stop_loop)

###################################
# PROCESSING SECTION
# Loop rotating the square until the mouse
# is clicked in the window.
# Branching code: none
# Looping code: while-loop, for-loop
###################################

# initialize the loop sentry
endloop = False

# main while-loop to draw the rotating square
while not endloop:
    # DEBUG CODE:
    # print a progress indicator
    print("starting loop",loop_count)
    # change to a new color each time through the loop
    newcolor = colors[loop_count%len(colors)]
    # DEBUG CODE:
    print("set the color to",newcolor)
    # change the color of the square
    square.color(newcolor)
    # increment the loop counter
    loop_count = loop_count + 1
    
    # rotate the square counterclockwise
    # DEBUG CODE:
    print("rotate counterclockwise")
    # for-loop to rotate the turtle counterclockwise in
    # 10 degree angle increments
    for a in range(37):
        # check to see if we need to end the loop
        if endloop: break
        # set the heading angle
        angle = 10  * a
        # DEBUG CODE:
        print(angle)
        # put the turtle on its new heading
        # this will cause it to rotate clockwise
        square.setheading(angle)

    # rotate the turtle clockwise
    print("rotate clockwise")
    for a in range(37):
        # check if the loop is ending
        if endloop: break
        # set the heading angle
        angle = -10 * a
        # DEBUG CODE:
        print(angle)
        # put the turtle on its new heading
        # this will cause it to rotate counterclockwise
        square.setheading(angle)

###################################   
# CLEANUP, TERMINATION, AND EXIT SECTION
###################################

# close the drawing window without user action
# (the done() function requires user action)
bye()
