###################################
# PROLOG SECTION
# shoot_ball.py
# A Turtle Graphics game to shoot at a bouncing ball target.
#
# In this game we combine the bouncing ball from bounce_ball.py,
# which we saw in an earlier lesson, with two other turtle objects:
#
# 1. shooter - this turtle never moves, but when the player clicks
#    on it, it will start the bullet moving upward toward (hopefully)
#    the moving ball.
# 2. bullet - starts from the position of the shooter when the player
#    clicks on the shooter.
#
# If the moving ball and bullet get close enough together, the player
# wins and the game is over.
#
# Tested with Python 3.7.9 on Windows 10
# Fri July 16, 2021
#
# Copyright ©. All rights reserved. NAF 2021.
#
# Possible future enhancements:
# Unresolved bugs:
###################################


###################################
# ENVIRONMENT SETUP SECTION
# Imports: turtle, random, winsound
# Note: winsound works only on the
# Windows platform.
###################################

# import the turtle routines
from turtle import *
# import the random number routines
from random import *
# import Windows sound routines
from winsound import *

###################################
# FUNCTION DEFINITION SECTION
# Definitions: shooter_clicked()
###################################

# function that is called when the shooter is clicked
# on the screen
def shooter_clicked(x,y):
    # get access to variable in the main program
    global bullet_moving
    print("shooter was clicked")
    # if the bullet is not moving yet, set
    # the flag to start it moving now.
    if not bullet_moving:
        # start the bullet moving up
        print("start the bullet moving")
        bullet_moving = True
    
###################################
# PROCESSING INITIALIZATION SECTION
###################################

# set the debug print flag
debug_print = False

#
# Set up the window.
#

# create the drawing window and
# set the graphics window dimensions
# to 600X400 centered at (0,0)
maxx = 300
maxy = 200
minx = -maxx
miny = -maxy
setup(width = 2 * maxx, height = 2 * maxy)

# set the window title
title("Shoot the Ball in Turtle Graphics")

# set the window background color
bgcolor('grey')

#
# Set up the turtle as a ball.
#

# create the turtle as a ball
ball=Turtle()
# hide the ball to begin with
ball.hideturtle()
# turn off drawing a line when the ball moves
# to see the ball's path, comment out this line
ball.penup()
# set the shape, size, and color of the ball
ball.shape("circle")
ball.shapesize(3,3,3)
ball.color("blue")
# Distance from center of ball to edge of window.
# Ball can't get any closer to the edge than this
# (at this point, the ball must "bounce").
bounce_point = 20

#
# Set up the ball to move.
#

# Pick a random starting location for the ball.
# Variables x and y are the x- and y-coordinates
# for the moving ball.
x = randrange(minx + bounce_point, maxx - bounce_point)
y = randrange(miny + bounce_point, maxy - bounce_point)

# Set the initial movement increments.
# This is how far the ball moves, in pixels
# each trip through the while-loop.
dx = 5
dy = 5

#
# Move the ball.
#

# Move the ball to its starting position.
# It is not visible yet.
ball.goto(x,y)

# Display the ball as it moves around the window.
ball.showturtle()

# draw the shooter turtle on the screen
# Note: this object never moves, but provides a place to
# click with the mouse to launch a bullet.
shooter = Turtle()
shooter.hideturtle()
shooter.penup()
shooter.goto(0,miny+20)
shooter.shape("square")
shooter.shapesize(2,2,2)
shooter.color("red")
shooter.showturtle()
# tell the player where to click to shoot the bullet
shooter.write("          Click the red shooter to shoot")

#
# Set up a mechanism to shoot by clicking
# on the shooter on the screen.
#
# When the shooter is clicked, shooter_clicked()
# is called, which sets bullet_moving to true.
shooter.onclick(shooter_clicked)


# set up the bullet as a small circular turtle
bullet = Turtle()
# set the bullet to its starting location
bullet.hideturtle()
bullet.penup()
bullet.shape("circle")
bullet.shapesize(1,1,1)
bullet.color("black")
bullet_y_start = miny+40
bx=0
bullet.goto(bx,bullet_y_start)

# initialize bullet movement constants
by = bullet_y_start
bdy = 7
bullet.showturtle()
             
# turn off the flag to indicate the bullet is in flight
# this means it starts out not moving
bullet_moving = False

# initialize the loop sentry variable
ball_was_hit=False

###################################
# PROCESSING SECTION
# Main loop:
# Keep moving the ball inside
# the window until it is hit
# by a bullet fired from
# the shooter.
#
# Much of the code in the while-loop is
# similar to what was in bounce_ball.py
#
# Branching code: if/else
# Looping code: while-loop
###################################

while not ball_was_hit:
     
    #
    # Calculate the possible new x-coordinate for the ball by
    # adding the move increment to the current x-coordinate value.
    #
    
    xx = x + dx

    #
    # If the new position is too close to the left-hand
    # edge of the window, set it to move in the opposite
    # direction and make a beep.
    #
    if xx < minx + bounce_point:
        # Limit x to the left-hand window edge.
        xx = minx + bounce_point
        # Reverse the x direction the ball is moving.
        dx = -dx
        # Sound a beep.
        MessageBeep(-1)
    
    #    
    # Same with the right-hand window.
    # 
    if xx > maxx - bounce_point:
        # Limit x to the right-hand window edge.
        xx = maxx - bounce_point
        # Reverse the x direction the ball is moving.
        dx = -dx
        # Sound a beep.
        MessageBeep(-1)

    #    
    # Calculate the possible new y-coordinate for the ball by
    # adding the move increment to the current y-coordinate value.
    #

    yy = y + dy

    #
    # Same with the y-coordinate and the top.
    # 
    if yy < miny + bounce_point:
        # limit y to the window top edge
        yy = miny + bounce_point
        # reverse the y direction the ball is moving
        dy = -dy
        # sound a beep
        MessageBeep(-1)

    #   
    # Same with the y-coordinate and the bottom.
    #
    if yy > maxy - bounce_point:
        # limit y to the bottom of the window
        yy = maxy - bounce_point
        # reverse the y direction
        dy = -dy
        # sound a beep
        MessageBeep(-1)

    #   
    # Set the new x- and y-coordinates for the center of the ball
    # using the values just calculated.
    #
    x = xx
    y = yy
    
    # Move the ball to its new location. This will cause the old
    # image of the ball in the window to be erased and a new one
    # to be drawn at its new position, thus producing an
    # animated view of the moving ball.
    
    ball.goto(x,y)

    # debug print
    if debug_print:
        print("DEBUG: ball at",x,y)

    # Once the ball has been moved, move the bullet, if
    # it is in flight.
    
    if bullet_moving:
        # possible new y-coordinate for the bullet
        by = by + bdy
        # check to see if the bullet is exiting the
        # window
        if by > maxy:
            # the bullet missed, start another bullet
            # by moving the bullet back to the starting
            # position
            by = bullet_y_start
            bullet_moving = False
            bullet.hideturtle()
            bullet.goto(bx,by)
            bullet.showturtle()

    # the bullet is moving, move it to its new location           
    bullet.goto(bx,by)
    
    # debug print
    if debug_print:
        print("DEBUG: bullet at",bx,by)

    # if the bullet is moving, see if it has hit the ball
    # (which means its distance to the center of the ball
    # is less than 20 pixels)
    if bullet_moving and bullet.distance(ball) < 20:
        print("You hit the ball!")
        # set a flag when the ball is hit so the
        # game animation stops
        ball_was_hit = True
        bullet.write("          Bullseye, game over!")

###################################   
# CLEANUP, TERMINATION, AND EXIT SECTION
###################################
        
# close the window when the mouse is clicked again
exitonclick()
